# Django settings for theatre project.

import os
from os.path import join
# import markdown

SITENAME = "India Theatre Forum"
DEBUG = True
TEMPLATE_DEBUG = DEBUG
JSON_DEBUG = DEBUG

THUMBNAIL_DEBUG = False

LOCAL_DEVELOPMENT = True
LOGGING_INTERCEPT_REDIRECTS = True
LOGGING_LOG_SQL = True
LOGGING_SHOW_METRICS = True
LOGGING_OUTPUT_ENABLED = True

PROJECT_PATH = os.path.dirname(__file__)
PROJECT_ROOT = PROJECT_PATH
SITE_CONFIG = join(PROJECT_ROOT, 'itf.json')

HAYSTACK_SITECONF = 'itf.search_sites'
HAYSTACK_SEARCH_ENGINE = 'whoosh'
HAYSTACK_WHOOSH_PATH = join(PROJECT_PATH, "../whoosh/itf_index")

CRISPY_TEMPLATE_PACK = 'uni_form'

CKEDITOR_MEDIA_PREFIX = "/static/ckeditor/"
CKEDITOR_UPLOAD_PATH = join(PROJECT_PATH, "static/upload/images/")

AUTH_PROFILE_MODULE = "itfprofiles.ItfProfile"

LOGIN_REDIRECT_URL = "/t/"

ADMINS = (
    ('Sanjay B', 'sanjaybhangar@gmail.com'),
)

INTERNAL_IPS = ('127.0.0.1',)

MANAGERS = ADMINS

#DATABASE_ENGINE = 'mysql'           # 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
#DATABASE_NAME = 'theatre'             # Or path to database file if using sqlite3.
#DATABASE_USER = 'root'             # Not used with sqlite3.
#DATABASE_PASSWORD = ''         # Not used with sqlite3.
#DATABASE_HOST = ''             # Set to empty string for localhost. Not used with sqlite3.
#DATABASE_PORT = ''             # Set to empty string for default. Not used with sqlite3.

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.mysql', # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': 'theatre',                      # Or path to database file if using sqlite3.
        'USER': 'root',                      # Not used with sqlite3.
        'PASSWORD': '',                  # Not used with sqlite3.
        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                      # Set to empty string for default. Not used with sqlite3.
    }
}



TWITTER_ID = "sachin_rt"

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
        'LOCATION': '127.0.0.1:11211',
    }
}

ERANG_SUBSCRIBE_URL = "http://a.organisedmail.com/nok.x?s=dGhlYXRyZWZvcnVt&ctid=YWRjZWUzOGVjMGYyZGUwNmYyOWFkZTk5ZWYyMjNjMjQ&emaddr=" 
# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

LANGUAGES = (
  ('en', 'English'),
  ('hi', 'Hindi'),
  ('kn', 'Kannada'),
  )

DEFAULT_LANGUAGE = 1

MARKITUP_PREVIEW_FILTER = ('markdown.markdown', {})

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = join(PROJECT_PATH, 'static')

TEMPLATE_CONTEXT_PROCESSORS = (
  "django.core.context_processors.auth",
  "django.core.context_processors.debug",
  "django.core.context_processors.i18n",
  "django.core.context_processors.media",
#  "multilingual.context_processors.multilingual",
  "frontpage.context_processors.menus"
  )

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = '/static/'
MARKITUP_SET = MEDIA_URL + 'markitup/sets/markdown/'

STATIC_URL = MEDIA_URL #FIXME: here due to weirdo code in django_markdown
# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/admin/media/'

# Make this unique, and don't share it with anybody.
# SECRET_KEY = 'fognneiw#s_o-#^tiny@^f-$x#1&29$3a37w5=kll57i!^uo@r'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
     'django.template.loaders.eggs.load_template_source',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
#    'multilingual.middleware.DefaultLanguageMiddleware',
    'debug_toolbar.middleware.DebugToolbarMiddleware',
#    'djangologging.middleware.LoggingMiddleware',
)

#SOLR_ROOT = '/home/sanj/satan/apache-solr-1.3.0/example/'
#SOLR_SCHEMA_PATH = SOLR_ROOT + 'solr/conf/schema.xml'
#SOLR_DATA_DIR = SOLR_ROOT + 'solr/data'

ROOT_URLCONF = 'urls'

TEMPLATE_DIRS = (
  join(PROJECT_PATH, 'templates'),
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',
    'django.contrib.comments',
    'django.contrib.markup',
    'registration',
#    'itfcore',
    'festival',
    'insidepages',
#    'erang',
    'erang_organised',
    'scriptbank',
    'bestpractices',
    'emailer',
    'itfprofiles',
    'pages',
    'tagging',
    'app',
    'padmavideos',
    'api',
#    'boxes',
    'frontpage',
#    'solango',
#    'multilingual',
#    'multilingual.flatpages',
    'django_extensions',
    'debug_toolbar',
    'sorl.thumbnail',
    'crispy_forms',
    'floppyforms',
#    'south',
    'user',
    'ckeditor',
    'fccv',
    'haystack',
    'markitup',
)

ACCOUNT_ACTIVATION_DAYS = 30

#overwrite default settings with local settings
try:
    from local_settings import *
except ImportError:
    pass

# Make this unique, creates random key first at first time.
try:
    SECRET_KEY
except NameError:
    SECRET_FILE = os.path.join(PROJECT_PATH, 'secret.txt')
    try:
        SECRET_KEY = open(SECRET_FILE).read().strip()
    except IOError:
        try:
            from random import choice
            SECRET_KEY = ''.join([choice('abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50)])
            secret = file(SECRET_FILE, 'w')
            secret.write(SECRET_KEY)
            secret.close()
        except IOError:
            Exception('Please create a %s file with random characters to generate your secret key!' % SECRET_FILE)


